;;; This source file is part of the Meta-CVS program, 
;;; which is distributed under the GNU license.
;;; Copyright 2002 Kaz Kylheku

(require "system")
(require "mapping")
(require "types")
(require "chatter")
(require "options")
(provide "rmap")

(defun mcvs-rmap (files)
  (in-sandbox-root-dir
    (let ((filemap (mapping-read *mcvs-map-local*)))
      (dolist (file files)
        (let* ((full-name (sandbox-translate-path file))
               (abs-name (canonicalize-path
                          (real-to-abstract-path full-name)))
               (entries (mapping-prefix-matches filemap abs-name)))
          (case (length entries)
            (0 (format t "~&UNMAPPED: ~a~%" file))
            (1 (format t "~&~a~%" (mapping-entry-id (car entries))))
            (otherwise (format t "~&DIRECTORY: ~a~%" full-name))))))))

(defun mcvs-rmap-wrapper (cvs-options cvs-command-options mcvs-args)
  (declare (ignore cvs-options cvs-command-options))
  (mcvs-rmap mcvs-args))

(defconstant *rmap-help*
  "Syntax:

  hoc rmap [ file list ]

Semantics:

  Rmap prints the reverse mapping (the F-* file) for the given files.
  It is handy to find files not under version control:

    find . \\( -type d -name HOC -prune \\) -o -type f -print | xargs hoc rmap")
