(require "mapping")
(provide "where")

(defun subst-entries-line (string tag branch?)
  "Substitute TAG name for the tag names in STRING which is a line
from the CVS/Entries file. If TAG is nil, then remove the sticky tag."
  ;; it seems "N" is not used in the Entries file
  ;;(if branch? "T" "N")
  (declare (ignore branch?))
  (let ((pos (position #\/ string :from-end t)))
    (if pos
        (concatenate 'string
                     (subseq string 0 (1+ pos))
                     (if tag
                         (concatenate 'string "T" tag)
                         ""))
        string)))

(defun subst-entries-file (entries-file tag branch?)
  "Substitute TAG for all tag names in the ENTRIES file (like
CVS/Entries).  A backup file is created for ENTRIES with .hoc
extension."
  (let ((old-entries-file (add-type-to-pathname entries-file "hoc")))
    (delete-file old-entries-file)
    (rename-file2 entries-file old-entries-file)
    (with-open-file (old old-entries-file :direction :input)
      (with-open-file (new entries-file :direction :output)
        (do ((line (read-line old nil) (read-line old nil)))
            ((null line))
          (format new "~A~%" (subst-entries-line line tag branch?)))))))

(defun subst-tag-file (tag-file tag branch?)
  "Create, overwrite or remove the TAG-FILE (usually CVS/Tag)
depending on if it exists and if TAG is nil. A backup file is created
for TAG-FILE with .hoc extension."
  (let ((old-tag-file (add-type-to-pathname tag-file "hoc")))
    (delete-file old-tag-file)
    (when (probe-file tag-file)
      (rename-file2 tag-file old-tag-file)))
  (unless (null tag)
    (with-open-file (new tag-file :direction :output)
      (if branch?
          (format new "T~A~%" tag)
          (format new "N~A~%" tag)))))

(defun subst-cvs-tag (cvs-dir tag branch?)
  "Set the current tag under CVS-DIR in the Entries and the Tag files
to TAG. TAG can be a BRANCH? tag."
  ;;(assert (pathname-directory-p cvs-dir))
  (subst-entries-file (merge-pathnames (make-pathname :name "Entries")
                                       cvs-dir)
                      tag branch?)
  (subst-tag-file (merge-pathnames (make-pathname :name "Tag")
                                   cvs-dir)
                  tag branch?))

;(defun subst-cvs-tag-recursively (dir tag branch?)
;  "Recursively update all cvs directories under DIR."
;  ;;(assert (pathname-directory-p dir))
;  (dolist (cvs-dir
;            (directory (merge-pathnames
;                        (make-pathname :directory
;                                       (list :relative :wild-inferiors "CVS"))
;                        dir)))
;    (subst-cvs-tag cvs-dir tag branch?)))

(defun version-branch (version)
  (let ((variant (ho-version:version-variant version)))
    (if (or (null variant) (zerop (length variant)))
        nil
        variant)))

(defun unstick ()
  ;; assume we are in *mcvs-dir*
  (subst-cvs-tag "CVS/"
                 (version-branch
                  (read-version-from-file *mcvs-version-name*))
                 t))

(defun mcvs-unstick ()
  (in-sandbox-root-dir
   (current-dir-restore 
    (chdir *mcvs-dir*)
    (unstick))))

(defun mcvs-unstick-wrapper (global-options command-options args)
  (declare (ignore global-options command-options))
  (when args
    (error "command takes no arguments."))
  (mcvs-unstick))

